use std::{process::ExitCode, str::FromStr};

use eframe::{self, egui::{self, Slider, TextEdit}};
use num_bigint_dig::BigUint;
mod rsa;

fn main() {
    let native_options = eframe::NativeOptions::default();
    eframe::run_native("Yeah", native_options, Box::new(|cc| Box::new(MyEguiApp::new(cc))));
}

#[derive(Default)]
struct MyEguiApp {
    pub_key: String,
    priv_key: String,
    n: BigUint,
    my_n: String,
    decrypted: String,
    encrypted: String,
}

impl MyEguiApp {
    fn new(cc: &eframe::CreationContext<'_>) -> Self {
        Self::default()
    }
}

impl eframe::App for MyEguiApp {
   fn update(&mut self, ctx: &egui::Context, frame: &mut eframe::Frame) {
        egui::containers::Window::new("Controls").show(&ctx, |ui| {
            if ui.button("Generate Keypair").clicked(){
                let (n, e, d) = rsa::generate_magics();
                self.n = n.clone();
                self.my_n = n.to_string().clone();
                self.pub_key = e.to_string();
                self.priv_key = d.to_string();
            }

            ui.text_edit_multiline(&mut self.my_n);
                if ui.button("Set N").clicked(){
                    self.n = BigUint::from_str(&self.my_n).unwrap();
                }

            if ui.button("Encrypt Message").clicked(){
                self.encrypted = rsa::encrypt(self.decrypted.clone(), BigUint::from_str(&self.pub_key).unwrap().clone(), self.n.clone());
            }

            if ui.button("Decrypt Message").clicked(){
                self.decrypted = rsa::decrypt(self.encrypted.clone(), &BigUint::from_str(&self.priv_key).unwrap(), &self.n);
            }
        });

        egui::containers::Window::new("Messages").show(&ctx, |ui|{
            ui.label("Encrypted");
            ui.text_edit_singleline(&mut self.encrypted);
            ui.label("Decrypted");
            ui.text_edit_singleline(&mut self.decrypted)
        });

        egui::containers::Window::new("Keys").show(&ctx, |ui|{
            ui.label("Public Key");
            ui.text_edit_multiline(&mut self.pub_key);
            ui.label("Private key");
            ui.text_edit_multiline(&mut self.priv_key);
        });
   }
}